unit uGUI_Main;
{$INCLUDE 'ChromePass.inc'}
{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  ActnList, ComCtrls, ShlObj;

type
  TFormMain = class(TForm)
    PanelCommon: TPanel;
    LabelPasswordDatabase: TLabel;
    ComboPasswordDatabase: TComboBox;
    btnSelectChromePasswordDatabase: TButton;
    ActionList1: TActionList;
    actSelectChromePasswordDatabase: TAction;
    actSelectExportFile: TAction;
    LabelExportFileName: TLabel;
    EditExportFileName: TEdit;
    btnSelectExportFile: TButton;
    pgctrlOperation: TPageControl;
    tabExport: TTabSheet;
    tabImport: TTabSheet;
    LabelExportPassword: TLabel;
    LabelExportPasswordRepeat: TLabel;
    EditExportPassword: TEdit;
    EditExportPasswordRepeat: TEdit;
    LabelImportPassword: TLabel;
    EditImportPassword: TEdit;
    CheckBoxEmptyChromeDatabase: TCheckBox;
    PanelButtons: TPanel;
    btnGo: TButton;
    actGo: TAction;
    opdlgPasswordDatabase: TOpenDialog;
    opdlgxPasswordDatabase: TFileOpenDialog;
    opdlgxExportFileName: TFileOpenDialog;
    opdlgExportFileName: TOpenDialog;
    procedure actSelectExportFileExecute(Sender: TObject);
    procedure actSelectChromePasswordDatabaseExecute(Sender: TObject);
    procedure actGoExecute(Sender: TObject);
    procedure ComboPasswordDatabaseDropDown(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    fHourglass: boolean;
    fFirstComboPasswordDatabaseDropDownDone: boolean;
    function SelectFileName(var FileName: string; OpenDialog: TOpenDialog; VistaOpenDialog: TFileOpenDialog): boolean;
    function GetChromePasswordDatabase(out FileName: string): boolean;
    function GetExportFileName(out FileName: string; MustExist: boolean): boolean;
    function GetExportPassword(out Password: string): boolean;
    function GetImportPassword(out Password: string): boolean;
    function CheckChromePassResult(ResultCode: integer): boolean;
    procedure SetHourglass(const Value: boolean);
    procedure FindFiles(const Path, WildCard: string; List: TStringList; Recursive: boolean);
    procedure FindChromeDatabases(List: TStringList);
  protected
    property Busy: boolean read fHourglass write SetHourglass;
  public
    procedure ExportLogins;
    procedure ImportLogins;
    procedure InitializeSettings;
    procedure ReadSettings;
    procedure WriteSettings;
  end;

var FormMain: TFormMain;

implementation

{$R *.DFM}

uses
  uChromePass, uConsts, uSystem;

{ TFormMain }

procedure TFormMain.actGoExecute(Sender: TObject);
begin
  if pgctrlOperation.ActivePage = tabExport then
    ExportLogins
  else if pgctrlOperation.ActivePage = tabImport then
    ImportLogins
  else
    ;
end;

procedure TFormMain.actSelectChromePasswordDatabaseExecute(Sender: TObject);
var
  FileName: string;
begin
  FileName := ComboPasswordDatabase.Text;
  if SelectFileName(FileName, opdlgPasswordDatabase, opdlgxPasswordDatabase) then
    ComboPasswordDatabase.Text := FileName;
end;

procedure TFormMain.actSelectExportFileExecute(Sender: TObject);
var
  FileName: string;
begin
  FileName := EditExportFileName.Text;
  if SelectFileName(FileName, opdlgExportFileName, opdlgxExportFileName) then
    EditExportFileName.Text := FileName;
end;

function TFormMain.CheckChromePassResult(ResultCode: integer): boolean;
begin
  Result := False;
  case ResultCode of
    EXITCODE_OK:
      begin
        MessageDlg('Operation completed successfully!', mtInformation, [mbOK], 0);
        Result := True;
      end;
    EXITCODE_NO_DATA:
      MessageDlg('No passwords found!', mtWarning, [mbOK], 0);
    else
      MessageDlg('Unspecified error occurred!', mtError, [mbOK], 0);
  end;
end;

procedure TFormMain.ComboPasswordDatabaseDropDown(Sender: TObject);
var
  L: TStringList;
begin
  if not fFirstComboPasswordDatabaseDropDownDone then
    begin
    fFirstComboPasswordDatabaseDropDownDone := True;
    Busy := True;
    try
      L := TStringList.Create;
      try
        FindChromeDatabases(L);
        ComboPasswordDatabase.Items.Assign(L);
      finally
        FreeAndNil(L);
        end;
    finally
      Busy := False;
      end;
    end;
end;

procedure TFormMain.ExportLogins;
var
  ChromePass: TChromePass;
  DatabaseFileName, ExportFileName, Password: string;
begin
  if GetChromePasswordDatabase(DatabaseFileName) then
    if GetExportFileName(ExportFileName, False) then
      if GetExportPassword(Password) then
        begin
        Busy := True;
        try
          ChromePass := TChromePass.Create;
          try
            ChromePass.ChromeFileName := DatabaseFileName;
            ChromePass.DataFileName := ExportFileName;
            ChromePass.Password := Password;
            if CheckChromePassResult(ChromePass.ExportPasswords) then
              WriteSettings;
          finally
            FreeAndNil(ChromePass);
            end;
        finally
          Busy := False;
          end;
        end;
end;

procedure TFormMain.FindChromeDatabases(List: TStringList);
var
  Dir: string;
begin
  if FindSpecialDirectory(CSIDL_APPDATA, Dir) then
    FindFiles(Dir, 'Login Data', List, True);
end;

procedure TFormMain.FindFiles(const Path, WildCard: string; List: TStringList; Recursive: boolean);
var
  SR: TSearchRec;
  PathWithDelimiter: string;
begin
  if Path = '' then
    PathWithDelimiter := ''
  else if (Length(Path) = 2) and (Path[2] = ':') then
    PathWithDelimiter := ''
  else
    PathWithDelimiter := IncludeTrailingPathDelimiter(Path);
  if FindFirst(PathWithDelimiter + WildCard, faAnyFile - faDirectory, SR) = 0 then
    try
      repeat
        if (SR.Attr and faDirectory) <> 0 then
          begin
          if Recursive then
            if (SR.Name <> '.') and (SR.Name <> '..') then
              FindFiles(PathWithDelimiter + SR.Name + PathDelim, WildCard, List, Recursive);
          end
        else
          List.Add(PathWithDelimiter + SR.Name);
      until FindNext(SR) <> 0;
    finally
      FindClose(SR);
    end;
  if Recursive then
    if FindFirst(PathWithDelimiter + '*.*', faDirectory, SR) = 0 then
      try
        repeat
          if (SR.Name <> '.') and (SR.Name <> '..') then
            FindFiles(PathWithDelimiter + SR.Name + PathDelim, WildCard, List, Recursive);
        until FindNext(SR) <> 0;
      finally
        FindClose(SR);
      end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  ReadSettings;
  pgctrlOperation.ActivePage := tabExport;
end;

function TFormMain.GetChromePasswordDatabase(out FileName: string): boolean;
begin
  FileName := ComboPasswordDatabase.Text;
  if FileName = '' then
    begin
    MessageDlg('You must provide a filename for Chrome password database!', mtError, [mbCancel], 0);
    Result := False;
    end
  else if FileExists(FileName) then
    Result := True
  else
    begin
    MessageDlg('File "' + FileName + '" does not exist!', mtError, [mbCancel], 0);
    Result := False;
    end;
end;

function TFormMain.GetExportFileName(out FileName: string; MustExist: boolean): boolean;
begin
  FileName := EditExportFileName.Text;
  if FileName = '' then
    begin
    MessageDlg('You must provide a filename for export file!', mtError, [mbCancel], 0);
    Result := False;
    end
  else if FileExists(FileName) then
    if MustExist then
      Result := True
    else
      Result := MessageDlg('File "' + FileName + '" already exists'#13#10'Are you sure you want to overwrite it?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) = mrYes
  else
    if MustExist then
      begin
      MessageDlg('File "' + FileName + '" does not exist!', mtError, [mbCancel], 0);
      Result := False;
      end
    else
      Result := True;
end;

function TFormMain.GetExportPassword(out Password: string): boolean;
begin
  Password := EditExportPassword.Text;
  if Password = EditExportPasswordRepeat.Text then
    Result := True
  else
    begin
    MessageDlg('The passwords are not the same!', mtError, [mbCancel], 0);
    Result := False;
    end;
end;

function TFormMain.GetImportPassword(out Password: string): boolean;
begin
  Password := EditImportPassword.Text;
  Result := True;
end;

procedure TFormMain.ImportLogins;
var
  ChromePass: TChromePass;
  DatabaseFileName, ExportFileName, Password: string;
  EmptyChromeDatabase: boolean;
begin
  if GetChromePasswordDatabase(DatabaseFileName) then
    if GetExportFileName(ExportFileName, True) then
      if GetImportPassword(Password) then
        begin
        EmptyChromeDatabase := CheckBoxEmptyChromeDatabase.Checked;
        if EmptyChromeDatabase then
          if MessageDlg('Are you sure you want to empty the Chrome password database?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) <> mrYes then
            Exit;
        Busy := True;
        try
          ChromePass := TChromePass.Create;
          try
            ChromePass.ChromeFileName := DatabaseFileName;
            ChromePass.DataFileName := ExportFileName;
            ChromePass.Password := Password;
            if CheckChromePassResult(ChromePass.ImportPasswords(EmptyChromeDatabase)) then
              WriteSettings;
          finally
            FreeAndNil(ChromePass);
            end;
        finally
          Busy := False;
          end;
        end;
end;

procedure TFormMain.InitializeSettings;
begin
  ComboPasswordDatabase.Text := '';
  EditExportFileName.Text := '';
  EditExportPassword.Text := '';
  EditExportPasswordRepeat.Text := '';
  EditImportPassword.Text := '';
  CheckBoxEmptyChromeDatabase.Checked := False;
end;

procedure TFormMain.ReadSettings;
begin
  InitializeSettings;
  {$MESSAGE WARN 'TODO'}
end;

function TFormMain.SelectFileName(var FileName: string; OpenDialog: TOpenDialog; VistaOpenDialog: TFileOpenDialog): boolean;
begin
  Result := False;
  if Win32MajorVersion >= 6 then
    begin
    VistaOpenDialog.FileName := FileName;
    if VistaOpenDialog.Execute then
      begin
      FileName := VistaOpenDialog.FileName;
      Result := True;
      end;
    end
  else
    begin
    OpenDialog.FileName := FileName;
    if OpenDialog.Execute then
      begin
      FileName := OpenDialog.FileName;
      Result := True;
      end;
    end;
end;

procedure TFormMain.SetHourglass(const Value: boolean);
const
  Cursors: array[boolean] of TCursor = (crDefault, crHourglass);
begin
  fHourglass := Value;
  Self.Enabled := not Value;
  Screen.Cursor := Cursors[Value];
end;

procedure TFormMain.WriteSettings;
begin
  {$MESSAGE WARN 'TODO'}
end;

end.
